var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import * as React from 'react';
import { defaultEditorOptions } from './config';
import CodeMirror from 'codemirror';
import './languages/log';
import './languages/simpleLog';
import 'codemirror/lib/codemirror.css';
import 'codemirror/addon/lint/lint.css';
import 'codemirror/addon/scroll/simplescrollbars.css';
import './style.scss';
import { getLinkMark, getLogMark } from './utils';
require('codemirror/mode/sql/sql');
require('codemirror/mode/python/python');
require('codemirror/mode/javascript/javascript');
require('codemirror/mode/properties/properties');
require('codemirror/addon/display/placeholder');
require('codemirror/addon/edit/matchbrackets');
require('codemirror/addon/scroll/simplescrollbars');
var CodeMirrorEditor = /** @class */ (function (_super) {
    __extends(CodeMirrorEditor, _super);
    function CodeMirrorEditor() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    CodeMirrorEditor.prototype.componentDidMount = function () {
        var ele = this._editorRef;
        if (!ele)
            return;
        var _a = this.props, value = _a.value, onChange = _a.onChange, onFocus = _a.onFocus, cursor = _a.cursor, options = _a.options, focusOut = _a.focusOut, cursorActivity = _a.cursorActivity, editorRef = _a.editorRef;
        var editorConfig = Object.assign({}, defaultEditorOptions, options);
        this._codeMirrorInstance = CodeMirror.fromTextArea(ele, editorConfig);
        var editorIns = this._codeMirrorInstance;
        this.renderTextMark();
        // 设置 cursor 位置
        if (cursor)
            editorIns.setCursor(cursor);
        editorIns.on('change', function (doc) {
            if (onChange) {
                onChange(value, doc.getValue(), doc);
            }
        });
        editorIns.on('focus', function (doc) {
            if (onFocus) {
                onFocus(value, doc.getValue(), doc);
            }
        });
        editorIns.on('blur', function (doc) {
            if (focusOut) {
                focusOut(value, doc.getValue(), doc);
            }
        });
        editorIns.on('cursorActivity', function (doc) {
            if (cursorActivity) {
                cursorActivity(value, doc.getValue(), doc);
            }
        });
        if (editorRef) {
            editorRef(editorIns);
        }
    };
    CodeMirrorEditor.fromTextArea = function (ele, options) {
        throw new Error("Method not implemented.");
    };
    // eslint-disable-next-line
    CodeMirrorEditor.prototype.UNSAFE_componentWillReceiveProps = function (nextProps) {
        var editorIns = this._codeMirrorInstance;
        var value = nextProps.value, sync = nextProps.sync, cursor = nextProps.cursor, cursorAlwaysInEnd = nextProps.cursorAlwaysInEnd, _a = nextProps.options, options = _a === void 0 ? {} : _a;
        if (options) {
            editorIns.setOption('readOnly', options.readOnly);
        }
        if (this.props.value !== value) {
            if (cursor)
                editorIns.setCursor(cursor);
            if (sync) {
                var scrollInfo = editorIns.getScrollInfo();
                /**
                 * 判断滚动条是不是在底部
                 */
                var isInBottom = (scrollInfo.top + scrollInfo.clientHeight) - scrollInfo.height > -10;
                console.log(isInBottom);
                if (!value) {
                    editorIns.setValue('');
                }
                else {
                    editorIns.setValue(value);
                }
                if (cursorAlwaysInEnd) {
                    editorIns.setCursor(editorIns.lineCount());
                }
                else if (!isInBottom) {
                    /**
                    * 不在底部并且不设置自动滚到底部，则滚到原来位置
                    */
                    editorIns.scrollTo(scrollInfo.left, scrollInfo.top);
                }
                else if (isInBottom) {
                    /**
                     * 在底部，则自动到底部
                     * 需要等setValue这个动作结束之后，再获取内容的高度。
                     */
                    Promise.resolve().then(function () {
                        var nowScrollInfo = editorIns.getScrollInfo();
                        editorIns.scrollTo(nowScrollInfo.left, nowScrollInfo.height);
                    });
                }
            }
            this.renderTextMark();
        }
    };
    CodeMirrorEditor.prototype.renderTextMark = function () {
        var editorIns = this._codeMirrorInstance;
        var marks = editorIns.getAllMarks();
        for (var _a = 0, marks_1 = marks; _a < marks_1.length; _a++) { // 重置marks
            var mark = marks_1[_a];
            mark.clear();
        }
        var value = editorIns.getValue();
        var linkMarks = [].concat(getLinkMark(value)).concat(getLogMark(value));
        for (var _i = 0; _i < linkMarks.length; _i++) {
            var mark = linkMarks[_i];
            editorIns.markText(editorIns.posFromIndex(mark.start), editorIns.posFromIndex(mark.end), { replacedWith: mark.node });
        }
    };
    CodeMirrorEditor.prototype.render = function () {
        var _this = this;
        var _a = this.props, className = _a.className, style = _a.style;
        var renderClass = "code-editor " + (className || '');
        var renderStyle = {
            position: 'relative'
        };
        Object.assign(renderStyle, style);
        return (React.createElement("div", { className: renderClass, style: renderStyle },
            React.createElement("textarea", { ref: function (e) { _this._editorRef = e; }, defaultValue: this.props.value || '' })));
    };
    return CodeMirrorEditor;
}(React.Component));
export default CodeMirrorEditor;
