var errorType;
(function (errorType) {
    errorType["info"] = "info";
    errorType["warning"] = "warning";
    errorType["error"] = "error";
})(errorType || (errorType = {}));
/**
 * Get Local time, format:  HH:mm:ss
 */
export function getLocalTime() {
    var date = new Date();
    // Get local hours, minutes, secondes
    return date.getHours() + ":" + date.getMinutes() + ":" + date.getSeconds();
}
export function createLinkMark(attrs) {
    return "#link#" + JSON.stringify(attrs) + "#link#";
}
export function getLinkMark(value) {
    var linkRegexp = /#link#(.+)#link#/g;
    var result = [];
    var indexObj;
    indexObj = linkRegexp.exec(value);
    while (indexObj) {
        var node = document.createElement('a');
        var attrs = JSON.parse(indexObj[1]);
        var keyAndValues = Object.entries(attrs);
        for (var _i = 0, keyAndValues_1 = keyAndValues; _i < keyAndValues_1.length; _i++) {
            var _a = keyAndValues_1[_i], _key = _a[0], _value = _a[1];
            node.setAttribute(_key, _value);
        }
        node.className = 'editor_custom_link';
        node.innerHTML = 'logDownload';
        result.push({
            start: indexObj.index,
            end: indexObj.index + indexObj[0].length,
            node: node
        });
        indexObj = linkRegexp.exec(value);
    }
    return result;
}
/**
 * 标记颜色
 * @param {string} text 标记文本
 * @param {string} type 标记类型
 */
export function createLogMark(text, type) {
    if (text === void 0) { text = ''; }
    if (type === void 0) { type = 'info'; }
    return "#log<" + type + ">log#" + text + "#log<" + type + ">log#";
}
export function getLogMark(value) {
    var result = [];
    function match() {
        var logRegexp = new RegExp("#log<(\\w*)>log#((.|\r\n|\n)*?)#log<(\\w*)>log#", 'g');
        var indexObj;
        indexObj = logRegexp.exec(value);
        /**
         * 循环正则来匹配相应格式字段
         */
        while (indexObj) {
            var text = indexObj[0].replace(/\r\n/g, '\n');
            /**
             * 包含标记字段的数组
             */
            var textArr = text.split('\n');
            var content = indexObj[2].replace(/\r\n/g, '\n');
            /**
             * 不包含标记字段的数组
             */
            var contentArr = content.split('\n');
            /**
             * 创建一个node，便于拷贝
             */
            var node = document.createElement('span');
            var type = indexObj[1];
            node.className = "c-editor--log__" + type;
            /**
             * 当前index偏移量
             */
            var offset = 0;
            for (var i = 0; i < textArr.length; i++) {
                var textItem = textArr[i];
                var contentItem = contentArr[i];
                var cloneNode = node.cloneNode(false); // 浅拷贝
                cloneNode.innerText = contentItem;
                result.push({
                    start: indexObj.index + offset,
                    end: indexObj.index + offset + textItem.length,
                    node: cloneNode
                });
                offset = offset + textItem.length + 1;
            }
            indexObj = logRegexp.exec(value);
        }
    }
    match();
    return result;
}
/**
 * dtlog日志构造器
 * @param {string} log 日志内容
 * @param {string} type 日志类型
 */
export function createLog(log, type) {
    if (type === void 0) { type = ''; }
    var now = getLocalTime();
    if (process.env.NODE_ENV == 'test') {
        now = 'test';
    }
    return "[" + now + "] <" + type + "> " + log;
}
export function createTitle(title) {
    if (title === void 0) { title = ''; }
    var baseLength = 15;
    var offsetLength = Math.floor(1.5 * title.length / 2);
    var arr = new Array(Math.max(baseLength - offsetLength, 5));
    var wraptext = arr.join('=');
    return "" + wraptext + title + wraptext;
}
